# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-10-20 08:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0007_user_token_lifetime'),
    ]

    operations = [
        migrations.AddField(
            model_name='sshpublickey',
            name='is_shared',
            field=models.BooleanField(default=False),
        ),
    ]
